@@ -46,7 +46,8 @@ func MakeIngress(dm *servingv1alpha1.DomainMapping, backendServiceName, hostName
 				return key == corev1.LastAppliedConfigAnnotation
 			}),
 			Labels: kmeta.UnionMaps(dm.Labels, map[string]string{
-				serving.DomainMappingLabelKey: dm.Name,
+				serving.DomainMappingUIDLabelKey:       string(dm.UID),
+				serving.DomainMappingNamespaceLabelKey: dm.Namespace,
 			}),
 			OwnerReferences: []metav1.OwnerReference{*kmeta.NewControllerRef(dm)},
 		},
@@ -56,20 +57,22 @@ func MakeIngress(dm *servingv1alpha1.DomainMapping, backendServiceName, hostName
 				Hosts:      []string{dm.Name},
 				Visibility: netv1alpha1.IngressVisibilityExternalIP,
 				HTTP: &netv1alpha1.HTTPIngressRuleValue{
-					Paths: append([]netv1alpha1.HTTPIngressPath{{
-						RewriteHost: hostName,
-						Splits: []netv1alpha1.IngressBackendSplit{{
-							Percent: 100,
-							AppendHeaders: map[string]string{
-								network.OriginalHostHeader: dm.Name,
-							},
-							IngressBackend: netv1alpha1.IngressBackend{
-								ServiceNamespace: dm.Namespace,
-								ServiceName:      backendServiceName,
-								ServicePort:      intstr.FromInt(80),
-							},
-						}},
-					}}, routeresources.MakeACMEIngressPaths(acmeChallenges, dm.GetName())...),
+					// The order of the paths is sensitive, always put tls challenge first
+					Paths: append(routeresources.MakeACMEIngressPaths(acmeChallenges, dm.GetName()),
+						[]netv1alpha1.HTTPIngressPath{{
+							RewriteHost: hostName,
+							Splits: []netv1alpha1.IngressBackendSplit{{
+								Percent: 100,
+								AppendHeaders: map[string]string{
+									network.OriginalHostHeader: dm.Name,
+								},
+								IngressBackend: netv1alpha1.IngressBackend{
+									ServiceNamespace: dm.Namespace,
+									ServiceName:      backendServiceName,
+									ServicePort:      intstr.FromInt(80),
+								},
+							}},
+						}}...),
 				},
 			}},
 		},
