@@ -31,18 +31,12 @@ import (
 	routecfg "knative.dev/serving/pkg/reconciler/route/config"
 
 	network "knative.dev/networking/pkg"
-	servingreconciler "knative.dev/serving/pkg/reconciler"
 	"knative.dev/serving/pkg/reconciler/nscert/config"
 )
 
-const (
-	controllerAgentName = "namespace-controller"
-)
-
 // NewController initializes the controller and is called by the generated code
 // Registers eventhandlers to enqueue events.
 func NewController(ctx context.Context, cmw configmap.Watcher) *controller.Impl {
-	ctx = servingreconciler.AnnotateLoggerWithName(ctx, controllerAgentName)
 	logger := logging.FromContext(ctx)
 	nsInformer := nsinformer.Get(ctx)
 	knCertificateInformer := kcertinformer.Get(ctx)
